# @package      hubzero-python
# @file         webconfig.py
# @author       David Benham <dbenham@purdue.edu>
# @copyright    Copyright (c) 2012 HUBzero Foundation, LLC.
# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
#
# Copyright (c) 2012 HUBzero Foundation, LLC.
#
# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
#
# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
# software: you can redistribute it and/or modify it under the terms of
# the GNU Lesser General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# HUBzero is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# HUBzero is a registered trademark of HUBzero Foundation, LLC.
#

import hubzero.config.hubzerositeconfig
import hubzero.data.db
import re
import os


def getDefaultSite():
	return hubzero.config.hubzerositeconfig.getHubzeroConfigOption("DEFAULT", "site")
	

def getWebConfigOption(optionName):
	
	defaultSite = getDefaultSite()
	wwwDocRoot = hubzero.config.hubzerositeconfig.getHubzeroConfigOption(defaultSite, "DocumentRoot")	
	
	# read config values from file
	file = open(wwwDocRoot + "/configuration.php")
	configValues = file.read()
	file.close()
	
	# hubconfig might still be around, if so, dump this file into our search string... we're gonna assume no variable
	if os.path.exists(wwwDocRoot + "/hubconfiguration.php"):
		file = open(wwwDocRoot + "/hubconfiguration.php")
		hubconfigFileContents = file.read()
		file.close()
		configValues += hubconfigFileContents
	
	# grep for right line
	regEx = "var\s*\$" + optionName + "\s*=\s*'(.*)';"
	m = re.search(regEx, configValues, re.MULTILINE | re.IGNORECASE)

	# return whatever we find, if not found, this will be none type
	if m:
		return m.group(1)
	else:
		return ""
		    

def getWebConfigDBPassword():
	return getWebConfigOption("password")


def getWebConfigDBUsername():
	return getWebConfigOption("user")


def getWebConfigDBName():
	return getWebConfigOption("db")


def _writeParamField(componentOption, NewParamValueDictionary):
	"""
	Take a dictionary, transform it to properly formatted params string for Joomla, and 
	update the params field of the jos_components table for the specified component
	"""
	# database config
	dbPW = getWebConfigDBPassword()
	dbUserName = getWebConfigDBUsername()
	dbName = getWebConfigDBName()	
	db =  hubzero.data.db.mySql("localhost", dbName, dbUserName, dbPW)

	parmString = ""
	for parm in NewParamValueDictionary:
		parmString += parm + "=" + NewParamValueDictionary[parm] + "\n"
		
	sql = "UPDATE jos_components set params = %s WHERE `option` = %s"
	data = (parmString, componentOption)
	db.execQuery(sql, data)


def _buildDictionary(componentOption):
	"""
	Put the parameter string of the database into a dictionary
	String follows the format:
	
	param1=value1
	param2=value2
	param3=value3
	
	"""
	# database config
	dbPW = getWebConfigDBPassword()
	dbUserName = getWebConfigDBUsername()
	dbName = getWebConfigDBName()	
	db =  hubzero.data.db.mySql("localhost", dbName, dbUserName, dbPW)
	
	# look up parm string and parse it on newline, for each line, parse it further based on *first* '='
	sql = "SELECT `params` from jos_components WHERE `option`=%s"
	data = (componentOption)
	paramsString = db.execSelectQueryScalar(sql, data)
	
	parms = paramsString.split()
	
	parmsDict = {}
	for parmline in parms:
		n,v = parmline.split("=", 1)
		parmsDict[n] = v
		
	return parmsDict
	

def getComponentParam(componentOption, parameterName):
	"""
	Get individual param value for a component and return to caller
	"""
	paramDict = _buildDictionary(componentOption)
	
	if paramDict.has_key(parameterName):
		return paramDict[parameterName]
	else:
		return None	


def addComponentParam(componentOption, parameterName, parameterValue, allowOverwrite=True):
	"""
	Add value to the param string in the jos_components table for the specified component.
	"""
	paramDict = _buildDictionary(componentOption)

	if allowOverwrite:
		paramDict[parameterName] = parameterValue
	else:
		if paramDict.has_key(parameterName):
			raise Exception("ERROR: addCompoentParam called for existing parameter with allowOverwrite = false")
		else:
			paramDict[parameterName] = parameterValue

	_writeParamField(componentOption, paramDict)	

	
def delComponentParam(componentOption, parameterName):
	"""
	Delete value from the param string in the in the jos_components table for the specified component
	"""
	paramDict = _buildDictionary(componentOption)
	del paramDict[parameterName]
	_writeParamField(componentOption, paramDict)
	
	
def setPluginEnabled(pluginFolder, pluginElement, enable):
	"""
	Enable or disable a plugin by setting the published field for the plugin in the jos_plugins table
	"""
	
	# database config
	dbPW = getWebConfigDBPassword()
	dbUserName = getWebConfigDBUsername()
	dbName = getWebConfigDBName()	
	db =  hubzero.data.db.mySql("localhost", dbName, dbUserName, dbPW)
		
	sql = "UPDATE jos_plugins set published = %s WHERE `folder` = %s AND `element` = %s"
	data = (enable, pluginFolder, pluginElement)
	db.execQuery(sql, data)	
